//
// teststar.c
//
// Program to test the starfield library.
//
// richard.cutting@virgin.net 
// http://www.netyaroze-europe.com/~rcutting
//
// Please feel free to use this code as you want to for your own non
// commercial use but please leave my name in the headers if you do.

#include <libps.h>              // PSX libraries

#include "pad.h"
#include "starfld.h"

// Define GPU packet work area

#define PACKET_SIZE 10000
#define PACKETMAX2 (PACKET_SIZE*24)
static PACKET packetArea[2][PACKETMAX2];

// Define ordering tables

#define OT_LENGTH		1

GsOT            WorldOT[2];
GsOT_TAG	OTTags[2][1<<OT_LENGTH];

// Variable to hold world viewpoint

GsRVIEW2 	WorldView;

#define FRAME_X		320
#define FRAME_Y		256

// Lighting

GsF_LIGHT	TheLights[3];

// For projection

int ProjectionDistance;	
//
// Initialise the graphics system must be called before any drawing etc
// can take place.
//

void init_graphics()
{
	GsInitGraph( FRAME_X, FRAME_Y, 4, 0, 0 );

	GsDefDispBuff( 0, 0, 0, FRAME_Y );

	GsInit3D();

	WorldOT[0].length = OT_LENGTH;
	WorldOT[0].org = OTTags[0];
	WorldOT[1].length = OT_LENGTH;
	WorldOT[1].org = OTTags[1];

	// Setup projection

        ProjectionDistance = 192;                       // screen to viewpoint distace

        GsSetProjection(ProjectionDistance);  

	// Set world viewpoint ( at origin )

	WorldView.vpx = 0;
        WorldView.vpy = 0;
        WorldView.vpz = -500;
	WorldView.vrx = 0;
	WorldView.vry = 0;
	WorldView.vrz = 0;
	WorldView.rz = 0;
	WorldView.super = WORLD;

	GsSetRefView2( &WorldView );

	// Setup light ( 3 flat lights )

	TheLights[0].vx = ONE; TheLights[0].vy = 0; TheLights[0].vz = 0;
	TheLights[0].r = 128; TheLights[0].g = 0; TheLights[0].b = 0;
	GsSetFlatLight(0, &TheLights[0]);
	TheLights[1].vx = 0; TheLights[1].vy = ONE; TheLights[1].vz = 0;
	TheLights[1].r = 0; TheLights[1].g = 128; TheLights[1].b = 0;
	GsSetFlatLight(1, &TheLights[1]);
	TheLights[2].vx = 0; TheLights[2].vy = 0; TheLights[2].vz = ONE;
	TheLights[2].r = 0; TheLights[2].g = 0; TheLights[2].b = 128;
	GsSetFlatLight(2, &TheLights[2]);	   
	
}

int check_controller( void )
{
	u_long pad;

	// Read controller pad.

	pad = PadRead();

	if ( pad & PADselect )
	{
		return( 0 );
	}

	return( 1 );
}

//
// Main entry point
//

void main()
{
	int vsync_time, active;

	GsOT *otp;

	// Initialise controllers

	PadInit();

	// Initialise graphics

	init_graphics();

	initialise_starfield( 250,			// 250 stars
			      3,			// 3 planes
			      STAR_SCROLL_RIGHT,	// scrolling right
			      0,			// don't change direction ever
			      1,			// animate every frame
			      OT_LENGTH );		// my OT length

	// GO !

        while( check_controller()  )
	{
		animate_starfield();

		// Get buffer in use.

                active = GsGetActiveBuff();

		// For speed...

		otp = &( WorldOT[active] );

		// Set GPU work area

		GsSetWorkBase( ( PACKET * ) packetArea[active] );

		// Clear object table

                GsClearOt( 0, 0, otp );

                // Update viewpoint

                GsSetRefView2( &WorldView );
	
		// Add objects into OT

		load_starfield_into_OT( otp );

		// Wait to draw...

                vsync_time = VSync( 0 );

                // Swap display buffer.

		GsSwapDispBuff();

		// Clear background.

		GsSortClear( 0, 0, 0, &( WorldOT[active] ) );

		// And draw objects.

		GsDrawOt( &( WorldOT[active] ) );

	}

	free_starfield();

	// Get out

	ResetGraph( 3 );
}
								
